@@ -4,6 +4,7 @@ Parse a junit report file into a family of objects
 from __future__ import unicode_literals
 
 import os
+import sys
 import xml.etree.ElementTree as ET
 import collections
 from .textutils import unicode_str
@@ -19,6 +20,23 @@ PASSED = "passed"  # the test completed successfully
 ABSENT = "absent"  # the test was known but not run/failed/skipped
 
 
+def clean_xml_attribute(element, attribute, default=None):
+    """
+    Get an XML attribute value and ensure it is legal in XML
+    :param element:
+    :param attribute:
+    :param default:
+    :return:
+    """
+
+    value = element.attrib.get(attribute, default)
+    if value:
+        value = value.encode("utf-8", errors="replace").decode("utf-8", errors="backslashreplace")
+        value = value.replace(u"\ufffd", "?")  # strip out the unicode replacement char
+
+    return value
+
+
 class ParserError(Exception):
     """
     We had a problem parsing a file
@@ -305,9 +323,15 @@ class Junit(object):
         :return:
         """
         self.filename = filename
+        if filename == "-":
+            # read the xml from stdin
+            stdin = sys.stdin.read()
+            xmlstring = stdin
+            self.filename = None
+
         self.tree = None
-        if filename is not None:
-            self.tree = ET.parse(filename)
+        if self.filename is not None:
+            self.tree = ET.parse(self.filename)
         elif xmlstring is not None:
             self._read(xmlstring)
         else:
@@ -355,10 +379,9 @@ class Junit(object):
             suitecount += 1
             cursuite = Suite()
             self.suites.append(cursuite)
-            suitename = suite.attrib.get("name", "suite-" + str(suitecount))
-            cursuite.name = suitename
-            if "package" in suite.attrib:
-                cursuite.package = suite.attrib["package"]
+            cursuite.name = clean_xml_attribute(suite, "name", default="suite-" + str(suitecount))
+            cursuite.package = clean_xml_attribute(suite, "package")
+
             cursuite.duration = float(suite.attrib.get("time", '0').replace(',', '') or '0')
 
             for element in suite:
@@ -396,9 +419,9 @@ class Junit(object):
 
                     testclass = cursuite[testcase.attrib["classname"]]
                     newcase = Case()
-                    newcase.name = testcase.attrib["name"]
+                    newcase.name = clean_xml_attribute(testcase, "name")
                     newcase.testclass = testclass
-                    newcase.duration = float(testcase.attrib.get("time", '0').replace(',','') or '0')
+                    newcase.duration = float(testcase.attrib.get("time", '0').replace(',', '') or '0')
                     testclass.cases.append(newcase)
 
                     # does this test case have any children?
@@ -439,5 +462,9 @@ class Junit(object):
         """
 
         doc = HTMLReport()
-        doc.load(self, os.path.basename(self.filename))
+        title = "Test Results"
+        if self.filename:
+            if os.path.exists(self.filename):
+                title = os.path.basename(self.filename)
+        doc.load(self, title=title)
         return str(doc)
